from sqlalchemy import *
from migrate import *
import migrate.changeset
import re
try:
    from sqlalchemy.dialects.mssql.pyodbc import MSDialect_pyodbc
except:
    from sqlalchemy.databases.mssql import MSSQLDialect_pyodbc as MSDialect_pyodbc

class Dictionaries(object):
    def __init__(self, meta):
        self.option = Table('option', meta,
                       Column('idoption', Integer, nullable=False, autoincrement=True, primary_key=True),
                       Column('type', String(20), nullable=False),
                       Column('text', String(200), nullable=False))


def upgrade(migrate_engine):
    meta = MetaData(bind = migrate_engine)
    # Upgrade operations go here. Don't create your own engine; use the engine
    # named 'migrate_engine' imported from migrate.
    dictionaries = Dictionaries(meta)
    for table in filter(lambda x: isinstance(x, Table), map(lambda x: getattr(dictionaries, x), dir(dictionaries))):
        basename = table.name
        for suffix in [ '1', '2' ]:
            table.name = basename + suffix
            table.create()
    conn = migrate_engine.connect()
    quote = '`'
    if type(conn.dialect) is MSDialect_pyodbc:
        quote = '"'
    for table in filter(lambda x: isinstance(x, Table), map(lambda x: getattr(dictionaries, x), dir(dictionaries))):
        basename = table.name[:-1]
        conn.execute('create view %s%s%s as select * from %s1' % (quote, basename, quote, basename))

def downgrade(migrate_engine):
    meta = MetaData(bind = migrate_engine)
    # Operations to reverse the above upgrade go here.
    conn = migrate_engine.connect()
    quote = '`'
    if type(conn.dialect) is MSDialect_pyodbc:
        quote = '"'
    dictionaries = Dictionaries(meta)
    for table in filter(lambda x: isinstance(x, Table), map(lambda x: getattr(dictionaries, x), dir(dictionaries))):
        basename = table.name
        conn.execute('drop view %s%s%s' % (quote, basename, quote))
    for table in filter(lambda x: isinstance(x, Table), map(lambda x: getattr(dictionaries, x), dir(dictionaries))):
        basename = table.name
        for suffix in [ '1', '2' ]:
            table.name = basename + suffix
            table.drop()











