from sqlalchemy import *
from migrate import *
from dictionaries.versions import views_helper


def getAgeRestrict(meta):
    agerestrict = Table('agerestrict', meta,
                        Column('alcoholpercent', Numeric(precision=4, scale=2), nullable=False, primary_key=True),
                        Column('age', Integer, nullable=True),
                        )
    return agerestrict


def getTimeRestrict(meta):
    timerestrict = Table('timerestrict', meta,
                         Column('timerestrictid', Integer, nullable=False, primary_key=True, autoincrement=True),
                         Column('alcoholpercent', Numeric(precision=4, scale=2), nullable=False),
                         Column('datebeg', Date, nullable=True),
                         Column('dateend', Date, nullable=True),
                         Column('daysbeg', Integer, nullable=True),
                         Column('daysend', Integer, nullable=True),
                         Column('timebeg', Time(timezone=False), nullable=True),
                         Column('timeend', Time(timezone=False), nullable=True),
                         )
    return timerestrict


def getTmcTables(meta):
    tmc1 = Table('tmc1', meta, autoload=True)
    tmc2 = Table('tmc2', meta, autoload=True)
    return [tmc1, tmc2]


def getRoleTables(meta):
    role1 = Table('role1', meta, autoload=True)
    role2 = Table('role2', meta, autoload=True)
    return [role1, role2]


# def createTable(table, migrate_engine):
# 	if not table.exists():
# 		table.create()
# 	views_helper.create_or_replace(migrate_engine, table)

# def dropTable(table):
# 	meta = MetaData(bind = migrate_engine)
# 	if table.exists():
# 		table.drop()
# 	views_helper.drop_if_exist(migrate_engine, table)


def upgrade(migrate_engine):
    # Upgrade operations go here. Don't create your own engine; use the engine
    # named 'migrate_engine' imported from migrate.
    meta = MetaData(bind=migrate_engine)

    for tmc in getTmcTables(meta):
        alcoholpercent = Column('alcoholpercent', Numeric(precision=4, scale=2), nullable=True)
        if not 'alcoholpercent' in tmc.columns:
            alcoholpercent.create(tmc)
    views_helper.update_dic_view(migrate_engine, 'tmc')

    for role in getRoleTables(meta):
        cashmanagement = Column('cashmanagement', Integer, nullable=True)
        if not 'cashmanagement' in role.columns:
            cashmanagement.create(role)
    views_helper.update_dic_view(migrate_engine, 'role')

    table = getTimeRestrict(meta)
    basename = table.name
    for suffix in ['1', '2']:
        table.name = basename + suffix
        if not table.exists():
            table.create()
    # update view
    views_helper.create_or_replace(migrate_engine, 'timerestrict', 'timerestrict1')

    table = getAgeRestrict(meta)
    basename = table.name
    for suffix in ['1', '2']:
        table.name = basename + suffix
        if not table.exists():
            table.create()
    # update view
    views_helper.create_or_replace(migrate_engine, 'agerestrict', 'agerestrict1')


def downgrade(migrate_engine):
    # Operations to reverse the above upgrade go here.
    meta = MetaData(bind=migrate_engine)
    for tmc in getTmcTables(meta):
        if 'alcoholpercent' in tmc.columns:
            tmc.c.alcoholpercent.drop()
    views_helper.update_dic_view(migrate_engine, 'tmc', tmc)

    for role in getRoleTables(meta):
        if 'cashmanagement' in role.columns:
            role.c.cashmanagement.drop()
    views_helper.update_dic_view(migrate_engine, 'role', role)

    table = getTimeRestrict(meta)
    basename = table.name
    for suffix in ['1', '2']:
        table.name = basename + suffix
        if table.exists():
            table.drop()
    # drop view
    views_helper.drop_if_exist(migrate_engine, basename)

    table = getAgeRestrict(meta)
    basename = table.name
    for suffix in ['1', '2']:
        table.name = basename + suffix
        if table.exists():
            table.drop()
    # drop view
    views_helper.drop_if_exist(migrate_engine, basename)
