from sqlalchemy import *
import migrate.changeset
from documents.versions import dump

def getDocTable(meta):
    return Table('document', meta, autoload=True)

def getShiftTable(meta):
    return Table('workshift', meta, autoload=True)

def getDocumentIndex(meta):
    return Index('documentunique', getDocTable(meta).c.workshiftid, getDocTable(meta).c.checknum, unique=True)

def upgrade(migrate_engine):
    meta = MetaData(bind = migrate_engine)
    dump.store(migrate_engine)
    # Upgrade operations go here. Don't create your own engine; use the engine
    # named 'migrate_engine' imported from migrate.
    if 'documentunique' not in [ind.name for ind in getDocTable(meta).indexes]:
        getDocumentIndex(meta).create()

def downgrade(migrate_engine):
    meta = MetaData(bind = migrate_engine)
    # Operations to reverse the above upgrade go here.

    if 'documentworkshiftidfk' in [key.name for key in getDocTable(meta).foreign_keys]:
        migrate.changeset.constraint.ForeignKeyConstraint([getDocTable(meta).c.workshiftid], [getShiftTable(meta).c.workshiftid], name = 'documentworkshiftidfk').drop()
    if 'documentunique' in [ind.name for ind in getDocTable(meta).indexes]:
        getDocumentIndex(meta).drop()

    migrate.changeset.constraint.ForeignKeyConstraint([getDocTable(meta).c.workshiftid], [getShiftTable(meta).c.workshiftid], name = 'documentworkshiftidfk', ondelete = 'CASCADE').create()
