# -*- coding: utf-8 -*-
from sqlalchemy import *
from migrate import *
from sqlalchemy.dialects.mysql.pyodbc import MySQLDialect
from documents.versions import dump
from artix.database.info import DBInfo

def dropTrigger(migrate_engine):
    migrate_engine.connect().execute("DROP TRIGGER IF EXISTS `triggerAfterInsert`;")

def createTrigger(migrate_engine):
    dbInfo = DBInfo()
    trigger_after_insert = """
CREATE TRIGGER triggerAfterInsert AFTER INSERT ON document
FOR EACH ROW
thisTrigger: BEGIN
    if (@TRIGGER_AFTER_INSERT = FALSE) AND (USER() = '%s@%s') THEN
        LEAVE thisTrigger;
    END IF;

    SET @factor:=(select moneyfactor FROM documenttype WHERE doctype = new.doctype);
    UPDATE workshift 
    SET sumsale = COALESCE(sumsale, 0) + if(new.doctype = 1, new.sumb, 0),
        sumsalecash = COALESCE(sumsalecash, 0) + if(new.doctype = 1, new.sumcash, 0),
        sumsalenoncash = COALESCE(sumsalenoncash, 0) + if(new.doctype = 1, new.sumnoncash, 0),
        sumsaleother = COALESCE(sumsalenoncash, 0) + if(new.doctype = 1, new.sumother, 0),
        sumgaincash = COALESCE(sumgaincash, 0) + if(new.doctype in (1,2,25), new.sumcash * @factor, 0),
        sumgainnoncash = COALESCE(sumgainnoncash, 0) + if(new.doctype in (1,2,25), new.sumnoncash * @factor, 0),
        sumrefund = COALESCE(sumrefund, 0) + if(new.doctype in (2,25), abs(new.sumb), 0),
        sumrefundcash = COALESCE(sumrefundcash, 0) + if(new.doctype in (2,25), new.sumcash, 0),
        sumrefundnoncash = COALESCE(sumrefundnoncash, 0) + if(new.doctype in (2,25), new.sumnoncash, 0),
        sumgain = COALESCE(sumgain, 0) + if(new.doctype in (1, 2, 25), new.sumb, 0),
        sumdrawer = COALESCE(sumdrawer, 0) + (new.sumcash * @factor),
        countsale = COALESCE(countsale, 0) + if(new.doctype = 1, 1, 0),
        countrefund = COALESCE(countrefund, 0) + if(new.doctype in (2, 25), 1, 0)
    WHERE workshiftid = new.workshiftid AND new.closed = 1;
END
""" % (dbInfo.user, dbInfo.host)

    migrate_engine.connect().execute(trigger_after_insert)

def createOldTrigger(migrate_engine):
    trigger_after_insert = """
CREATE TRIGGER triggerAfterInsert AFTER INSERT ON document
FOR EACH ROW BEGIN
    SET @factor:=(select moneyfactor FROM documenttype WHERE doctype = new.doctype);
    UPDATE workshift 
    SET sumsale = COALESCE(sumsale, 0) + if(new.doctype = 1, new.sumb, 0),
        sumsalecash = COALESCE(sumsalecash, 0) + if(new.doctype = 1, new.sumcash, 0),
        sumsalenoncash = COALESCE(sumsalenoncash, 0) + if(new.doctype = 1, new.sumnoncash, 0),
        sumsaleother = COALESCE(sumsalenoncash, 0) + if(new.doctype = 1, new.sumother, 0),
        sumgaincash = COALESCE(sumgaincash, 0) + if(new.doctype in (1,2,25), new.sumcash * @factor, 0),
        sumgainnoncash = COALESCE(sumgainnoncash, 0) + if(new.doctype in (1,2,25), new.sumnoncash * @factor, 0),
        sumrefund = COALESCE(sumrefund, 0) + if(new.doctype in (2,25), abs(new.sumb), 0),
        sumrefundcash = COALESCE(sumrefundcash, 0) + if(new.doctype in (2,25), new.sumcash, 0),
        sumrefundnoncash = COALESCE(sumrefundnoncash, 0) + if(new.doctype in (2,25), new.sumnoncash, 0),
        sumgain = COALESCE(sumgain, 0) + if(new.doctype in (1, 2, 25), new.sumb, 0),
        sumdrawer = COALESCE(sumdrawer, 0) + (new.sumcash * @factor),
        countsale = COALESCE(countsale, 0) + if(new.doctype = 1, 1, 0),
        countrefund = COALESCE(countrefund, 0) + if(new.doctype in (2, 25), 1, 0)
    WHERE workshiftid = new.workshiftid AND new.closed = 1;
END
"""

    migrate_engine.connect().execute(trigger_after_insert)

def upgrade(migrate_engine):
    dump.store(migrate_engine)
    meta = MetaData(bind=migrate_engine)

    if isinstance(migrate_engine.connect().dialect, MySQLDialect):
        dropTrigger(migrate_engine)

    if isinstance(migrate_engine.connect().dialect, MySQLDialect):
        createTrigger(migrate_engine)

def downgrade(migrate_engine):
    meta = MetaData(bind=migrate_engine)

    if isinstance(migrate_engine.connect().dialect, MySQLDialect):
        dropTrigger(migrate_engine)
        
    if isinstance(migrate_engine.connect().dialect, MySQLDialect):
        createOldTrigger(migrate_engine)
